<?php
namespace Drupal\ms_membership\Tests;

/**
 * Checks behavior of Membership Suite
 *
 * @group ms_membership
 */
class MsMembershipTestCase extends \Drupal\simpletest\WebTestBase {

  protected $profile = 'standard';

  protected $admin_user;

  protected $regular_user;

  /**
   * @return array
   */
  public static function getInfo() {
    return [
      'name' => 'Membership Suite',
      'description' => 'Checks behavior of Membership Suite',
      'group' => 'MoneySuite',
    ];
  }

  public /**
   * Enable module.
   */
  function setUp() {
    // Enable all modules needed
    parent::setUp('ms_membership');

    // Create our users
    $this->admin_user = (object) $this->drupalCreateUser([
      'administer ms_membership plans',
      'administer ms_membership purchases',
      'administer moneysuite settings',
      'access administration pages',
    ]);
    $this->regular_user = $this->drupalCreateUser([
      'purchase ms_membership products',
      'modify own ms_membership purchases',
      'renew own ms_membership purchases',
      'view own ms_membership purchases',
      'have multiple ms_membership purchases',
      'cancel own ms_membership purchases',
    ]);
    // Login the admin
    /** @noinspection PhpParamsInspection */
    $this->drupalLogin($this->admin_user);

    // Save the settings
    $edit = [
      'ms_membership_force_signup' => TRUE,
      'ms_membership_show_free_special_urls' => TRUE,
      'ms_membership_skip_registration' => TRUE,
      'ms_membership_bypass_checkout_free' => TRUE,
      'ms_membership_show_membership_price' => TRUE,
      'ms_membership_user_create_time' => 'after_payment',
      'ms_membership_refund_remove' => TRUE,
    ];
    $this->drupalPost('admin/moneysuite/ms_membership', $edit, 'Save configuration');
  }

  public /**
   * Check that Membership Plans work how they should.
   */
  function testMembershipPlans() {
    // Login the admin
    $this->drupalLogin($this->admin_user);

    // Create a membership plan.
    $plan_vars = [
      'name' => 'Premium Plan',
      'sku' => 'ms_membership_1',
      'description' => 'This is the best membership',
      'recurring' => FALSE,
      'expiring_mail_days' => 3,
      'show_registration' => TRUE,
      'show_account' => TRUE,
      'fixed_date' => FALSE,
      'start_grant_roles[3]' => TRUE,
      'expire_remove_roles[3]' => TRUE,
      'main_amount' => 9.95,
      'main_length' => 5,
      'main_unit' => 'D',
    ];
    $this->drupalPost('admin/structure/ms_products/ms_membership/add', $plan_vars, 'Save');

    // Check that the plan shows on the page
    $this->assertText('Membership Plan Saved: Premium Plan.', 'The plan was saved correctly.');

    // Edit a membership plan
    $plan_vars = ['name' => 'Silver Plan'];
    $this->drupalPost('admin/structure/ms_products/list/ms_membership_1/edit', $plan_vars, 'Save');

    // Check that the plan shows on the page
    $this->assertText('Membership Plan Saved: Silver Plan.', 'The plan was updated correctly.');

    // Create another membership plan
    $plan_vars = [
      'name' => 'Gold Plan',
      'sku' => 'ms_membership_2',
      'description' => 'This is a good membership',
      'recurring' => TRUE,
      'show_registration' => FALSE,
      'show_account' => TRUE,
      'fixed_date' => FALSE,
      'start_grant_roles[3]' => TRUE,
      'expire_remove_roles[3]' => TRUE,
      'main_amount' => 7.95,
      'main_length' => 10,
      'main_unit' => 'D',
    ];
    $this->drupalPost('admin/structure/ms_products/ms_membership/add', $plan_vars, 'Save');

    // Check that the plan shows on the page
    $this->assertText('Membership Plan Saved: Gold Plan.', 'The plan was saved correctly.');

    /**
     * Check the user registration page to make sure plans are showing when they should be
     */
    $this->drupalLogout();
    $this->drupalGet('user/register');

    $this->assertText('Silver Plan', 'The Silver Plan is showing.');
    $this->assertNoText('Gold Plan', 'The Gold Plan is not showing.');

    /**
     * Check that Memberships work as the should
     */
    // Login the admin
    $this->drupalLogin($this->admin_user);

    // Add a membership to a user manually
    $edit = [
      'username' => $this->regular_user->name,
      'sku' => 'ms_membership_1',
    ];
    $this->drupalPost('admin/structure/ms_products/ms_membership/purchases/add', $edit, 'Save');

    $account = user_load($this->regular_user->uid);

    // Check that the role is granted when it should be
    $this->assertTrue(in_array('administrator', array_values($account->roles)), 'The role was granted successfully.');

    // Check the Memberships page to make sure it shows
    $this->drupalGet('admin/structure/ms_products/ms_membership/purchases/list');
    $this->assertText('Silver Plan', 'Membership Showing');
    $this->assertText('Active - Payments Completed', 'Status correct');

    // Check that the Membership shows in the User Account
    $this->drupalLogin($this->regular_user);
    $this->drupalGet('user/' . $this->regular_user->uid . '/ms_membership');
    $this->assertText('Silver Plan', 'Membership Record Showing in Account');

    // Change the membership to expired
    $this->drupalLogin($this->admin_user);
    $membership = ms_products_purchase_load($account->ms_memberships[0]->id);
    $m_plan = ms_products_plan_load($membership->sku);

    ms_products_change_purchase_status($membership->id, 'expired');
    ms_membership_change_roles($membership->uid, $m_plan->expire_grant_roles, $m_plan->expire_remove_roles);

    // Check the Memberships page to make sure it shows as expired
    $this->drupalGet('admin/people/members');
    $this->assertText('Silver Plan - Expired / Inactive', 'Membership Shows as Expired');

    // Make sure the role was removed
    $account = user_load($membership->uid, TRUE);
    $this->assertTrue(!in_array('administrator', array_values($account->roles)), 'The role was removed successfully.');
  }

  // @todo - Test adding membership


  public /**
   * Check various pages in the module for errors
   */
  function testAdminPages() {
    // Login the admin
    $this->drupalLogin($this->admin_user);
    $this->drupalGet('admin/structure/ms_products/ms_membership/purchases/add');
    $this->drupalGet('admin/structure/ms_products/ms_membership/purchases/add');
  }

  public function importMembershipPlans() {
    $ms_products_plan = new MsProductsPlan();
    $ms_products_plan->disabled = FALSE;
    $ms_products_plan->api_version = 1;
    $ms_products_plan->bundle = 'ms_membership';
    $ms_products_plan->sku = 'ms_membership_1';
    $ms_products_plan->uid = 1;
    $ms_products_plan->name = 'Premium Membership';
    $ms_products_plan->cart_type = 'recurring';
    $ms_products_plan->description = 'Example of a Recurring Membership. This will recur automatically until it is canceled by the user.';
    $ms_products_plan->signup_mail_subject = 'Thank you for Signing Up!';
    $ms_products_plan->signup_mail_body = 'Dear [ms_core_order:customerName],
  You have purchased the [ms_products_plan:name] membership for [ms_core_payment:paymentAmount] on [date:long], and your account access has been upgraded. Thank you!

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->eot_mail_subject = 'Your Membership has ended';
    $ms_products_plan->eot_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership has expired or been cancelled. The following changes have been made to your account:


  To renew your membership, please click the following link: [ms_products_purchase:renew-link:absolute]

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->cancel_mail_subject = 'Membership Cancelled';
    $ms_products_plan->cancel_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership has been cancelled and will not automatically renew anymore. Access will be removed at the end of the term.

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->modify_mail_subject = 'Thank You!';
    $ms_products_plan->modify_mail_body = 'Dear [ms_core_order:customerName],
  Your membership has been successfully extended.

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->expiring_mail_subject = 'Your Membership is Expiring Soon!';
    $ms_products_plan->expiring_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership will expire on [ms_products_purchase:expiration:short]. To renew your membership, click the following link: [ms_products_purchase:renew-link:absolute]

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->expiring_mail_days = 0;
    $ms_products_plan->weight = 1;
    $ms_products_plan->recurring_schedule = [
      'total_occurrences' => '0',
      'main_amount' => '9.95',
      'main_length' => '1',
      'main_unit' => 'M',
      'has_trial' => TRUE,
      'trial_amount' => '0.00',
      'trial_length' => '1',
      'trial_unit' => 'W',
      'fixed_date' => 0,
      'fixed_date_discount' => 1,
      'fixed_date_type' => 'M',
      'fixed_date_string' => '',
    ];
    $ms_products_plan->modify_options = [
      'ms_membership_3' => 0,
      'ms_membership_2' => 0,
    ];
    $ms_products_plan->allow_roles = [
      1 => 0,
      2 => 0,
      3 => 0,
      4 => 0,
      5 => 0,
    ];
    $ms_products_plan->deny_roles = [
      1 => 0,
      2 => 0,
      3 => 0,
      4 => 0,
      5 => 0,
    ];
    $ms_products_plan->expire_when = 'subscr_eot';
    $ms_products_plan->data = [
      'override_settings' => [],
      'show_registration' => 1,
      'show_account' => 1,
      'start_grant_roles' => [
        5 => '5',
        3 => 0,
        4 => 0,
      ],
      'start_remove_roles' => [
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'expire_grant_roles' => [
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'expire_remove_roles' => [
        5 => '5',
        3 => 0,
        4 => 0,
      ],
      'register_urls' => '',
      'allow_multiple' => 0,
      'shippable' => 1,
      'allow_roles' => [
        1 => 0,
        2 => 0,
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'deny_roles' => [
        1 => 0,
        2 => 0,
        3 => 0,
        4 => 0,
        5 => 0,
      ],
    ];
    $ms_products_plan->save();

    $ms_products_plan = new MsProductsPlan();
    $ms_products_plan->disabled = FALSE;
    $ms_products_plan->api_version = 1;
    $ms_products_plan->bundle = 'ms_membership';
    $ms_products_plan->sku = 'ms_membership_2';
    $ms_products_plan->uid = 1;
    $ms_products_plan->name = 'Silver Membership';
    $ms_products_plan->cart_type = 'cart';
    $ms_products_plan->description = 'Example of a Non-Recurring Membership. This type of membership will expire after a set period of time, after which it will need to be manually renewed from the User Account.';
    $ms_products_plan->signup_mail_subject = 'Thank you for Signing Up!';
    $ms_products_plan->signup_mail_body = 'Dear [ms_core_order:customerName],
  You have purchased the [ms_products_plan:name] membership for [ms_core_payment:paymentAmount] on [date:long], and your account access has been upgraded. Thank you!

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->eot_mail_subject = 'Your Membership has ended';
    $ms_products_plan->eot_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership has expired or been cancelled. The following changes have been made to your account:


  To renew your membership, please click the following link: [ms_products_purchase:renew-link:absolute]

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->cancel_mail_subject = 'Membership Cancelled';
    $ms_products_plan->cancel_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership has been cancelled and will not automatically renew anymore. Access will be removed at the end of the term.

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->modify_mail_subject = 'Thank You!';
    $ms_products_plan->modify_mail_body = 'Dear [ms_core_order:customerName],
  Your membership has been successfully extended.

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->expiring_mail_subject = 'Your Membership is Expiring Soon!';
    $ms_products_plan->expiring_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership will expire on [ms_products_purchase:expiration:short]. To renew your membership, click the following link: [ms_products_purchase:renew-link:absolute]

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->expiring_mail_days = 0;
    $ms_products_plan->weight = 0;
    $ms_products_plan->recurring_schedule = [
      'total_occurrences' => 1,
      'main_amount' => '20.00',
      'main_length' => '2',
      'main_unit' => 'M',
      'has_trial' => FALSE,
      'trial_amount' => 0,
      'trial_length' => 0,
      'trial_unit' => '',
      'fixed_date' => 0,
      'fixed_date_discount' => 1,
      'fixed_date_type' => 'M',
      'fixed_date_string' => '',
    ];
    $ms_products_plan->modify_options = [
      'ms_membership_3' => 0,
      'ms_membership_1' => 0,
    ];
    $ms_products_plan->allow_roles = [
      1 => 0,
      2 => 0,
      3 => 0,
      4 => 0,
      5 => 0,
    ];
    $ms_products_plan->deny_roles = [
      1 => 0,
      2 => 0,
      3 => 0,
      4 => 0,
      5 => 0,
    ];
    $ms_products_plan->expire_when = 'subscr_eot';
    $ms_products_plan->data = [
      'override_settings' => [],
      'show_registration' => 0,
      'show_account' => 1,
      'start_grant_roles' => [
        5 => '5',
        3 => 0,
        4 => 0,
      ],
      'start_remove_roles' => [
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'expire_grant_roles' => [
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'expire_remove_roles' => [
        5 => '5',
        3 => 0,
        4 => 0,
      ],
      'register_urls' => '',
      'allow_multiple' => 1,
      'shippable' => 0,
      'allow_roles' => [
        1 => 0,
        2 => 0,
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'deny_roles' => [
        1 => 0,
        2 => 0,
        3 => 0,
        4 => 0,
        5 => 0,
      ],
    ];
    $ms_products_plan->save();

    $ms_products_plan = new MsProductsPlan();
    $ms_products_plan->disabled = FALSE;
    $ms_products_plan->api_version = 1;
    $ms_products_plan->bundle = 'ms_membership';
    $ms_products_plan->sku = 'ms_membership_3';
    $ms_products_plan->uid = 1;
    $ms_products_plan->name = 'Gold Membership';
    $ms_products_plan->cart_type = 'recurring';
    $ms_products_plan->description = 'This is a fixed date membership.';
    $ms_products_plan->signup_mail_subject = 'Thank you for Signing Up!';
    $ms_products_plan->signup_mail_body = 'Dear [ms_core_order:customerName],
  You have purchased the [ms_products_plan:name] membership for [ms_core_payment:paymentAmount] on [date:long], and your account access has been upgraded. Thank you!

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->eot_mail_subject = 'Your Membership has ended';
    $ms_products_plan->eot_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership has expired or been cancelled. The following changes have been made to your account:


  To renew your membership, please click the following link: [ms_products_purchase:renew-link:absolute]

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->cancel_mail_subject = 'Membership Cancelled';
    $ms_products_plan->cancel_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership has been cancelled and will not automatically renew anymore. Access will be removed at the end of the term.

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->modify_mail_subject = 'Thank You!';
    $ms_products_plan->modify_mail_body = 'Dear [ms_core_order:customerName],
  Your membership has been successfully extended.

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->expiring_mail_subject = 'Your Membership is Expiring Soon!';
    $ms_products_plan->expiring_mail_body = 'Dear [ms_core_order:customerName],
  Your [ms_products_plan:name] membership will expire on [ms_products_purchase:expiration:short]. To renew your membership, click the following link: [ms_products_purchase:renew-link:absolute]

  Sincerely,
  [site:name]
  [site:url]';
    $ms_products_plan->expiring_mail_days = 0;
    $ms_products_plan->weight = 0;
    $ms_products_plan->recurring_schedule = [
      'total_occurrences' => '0',
      'main_amount' => '8.95',
      'main_length' => 1,
      'main_unit' => 'Y',
      'has_trial' => TRUE,
      'trial_amount' => 1.42,
      'trial_length' => 58,
      'trial_unit' => 'D',
      'fixed_date' => 1,
      'fixed_date_discount' => 1,
      'fixed_date_type' => 'Y',
      'fixed_date_string' => 'April 15',
    ];
    $ms_products_plan->modify_options = [
      'ms_membership_2' => 0,
      'ms_membership_1' => 0,
    ];
    $ms_products_plan->allow_roles = [
      1 => 0,
      2 => 0,
      3 => 0,
      4 => 0,
      5 => 0,
    ];
    $ms_products_plan->deny_roles = [
      1 => 0,
      2 => 0,
      3 => 0,
      4 => 0,
      5 => 0,
    ];
    $ms_products_plan->expire_when = 'subscr_eot';
    $ms_products_plan->data = [
      'override_settings' => [],
      'show_registration' => 1,
      'show_account' => 0,
      'start_grant_roles' => [
        5 => '5',
        3 => 0,
        4 => 0,
      ],
      'start_remove_roles' => [
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'expire_grant_roles' => [
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'expire_remove_roles' => [
        5 => '5',
        3 => 0,
        4 => 0,
      ],
      'register_urls' => '',
      'allow_multiple' => 0,
      'shippable' => 1,
      'allow_roles' => [
        1 => 0,
        2 => 0,
        3 => 0,
        4 => 0,
        5 => 0,
      ],
      'deny_roles' => [
        1 => 0,
        2 => 0,
        3 => 0,
        4 => 0,
        5 => 0,
      ],
    ];
    $ms_products_plan->save();
  }

}
